// Linux version

#include <stdio.h>
#include <stdlib.h>

#include <time.h>
#include <sys/time.h>

#include <phidget21.h>

// This struct gets passed through the "usrptr" part of the Phidgets API
struct userStruct {
   struct timespec start;
   FILE *outputFile;
};

// Create a listener function for changes in thermocouple temperature
int TemperatureDataHandler(CPhidgetTemperatureSensorHandle device, 
                           void *userptr, int index, double data)
{
    double ambientTemperature;

    struct timespec now;
    double timeDifference;
    clock_gettime(CLOCK_REALTIME, &now);

    // Extract the passed user data structure, and the file pointer within it
    struct userStruct *passedData = (struct userStruct *) userptr;
    FILE *file = passedData->outputFile;

    // Get the board temperature
	CPhidgetTemperatureSensor_getAmbientTemperature(device, &ambientTemperature);

    // Print dot to the screen to indicate one more data point recorded
    printf(".");
    fflush(stdout);

    // Sums the seconds plus nanoseconds
    // Translates to seconds by using 10^9 nanoseconds per second
    timeDifference = ((now.tv_sec - passedData->start.tv_sec) + 
        (float) (now.tv_nsec - passedData->start.tv_nsec) / 1000000000.0);

    fprintf(file, "%6f, %6f, %6f\n", timeDifference, data, ambientTemperature);
    fflush(file);

    return 0;
}

int main(int argc, char* argv[]) {

    int result;
    char character;

    // Obtain an intial time from which to generate timestamps
    struct timespec start;
    clock_gettime(CLOCK_REALTIME, &start);

    // Clear and open the file for writing
    FILE *file = fopen("temperature_data.csv","w");

    struct userStruct userData;
    userData.start = start;
    userData.outputFile = file;

    // Print the header at the top of the file first thing
    fprintf(file, "Time,Thermocouple-Temperature,Ambient-Board-Temperature\n");

    // Create the Phidget software object, hook in the event function, and open the object
    CPhidgetTemperatureSensorHandle device = 0;
    CPhidgetTemperatureSensor_create(&device);
    CPhidgetTemperatureSensor_set_OnTemperatureChange_Handler(device, 
            TemperatureDataHandler, (void *) &userData); 
    CPhidget_open((CPhidgetHandle)device, -1);
    result = CPhidget_waitForAttachment((CPhidgetHandle)device, 10000);
    if (result) {
        printf("No Device!\n");
        return 0;
    }

    // Set the thermocouple type and trigger (0 = record all data)           
    CPhidgetTemperatureSensor_setTemperatureChangeTrigger(device, 0, 0);
    CPhidgetTemperatureSensor_setThermocoupleType(device, 0, 
            PHIDGET_TEMPERATURE_SENSOR_K_TYPE);

    printf("Press Enter to end\n");
    getchar();

    CPhidget_close((CPhidgetHandle)device);
    CPhidget_delete((CPhidgetHandle)device);
    fclose(file);
    return 0;
}

//==============================================================================
// EOF
//==============================================================================


